                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Authors note:   |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/

#include <afxwin.h>
#include <atlbase.h>
#include <afxtempl.h>

#include <GL/GL.h>
#include <GL/GLU.h>

#include "3D/Resolution.h"
#include "OpenGL/glExtensions.h"
#include "OpenGL/Texture.h"
#include "Common/CPU.h"

#include "3D/Matrix.h"
#include "3D/Particles.h"

#include "OpenGL/SpotLight.h"


#define VK_ENTER 13

/* ################################################################################################################### */

class OpenGLWindow : public CFrameWnd {
public:
    OpenGLWindow();

	void DrawFrame();
protected:
	HDC hdc;
	HGLRC hrc;

	CRect windowedRect;
	int Width, Height;
	int Bpp;
	int Refresh;
	bool fullScreen;
	bool captureMouse;
	bool windowMoved;
	int middleX, middleY;

	bool Key[256];
	bool leftButton;
	float mouseSensibility;
	bool invertMouse;
	int mouseX, mouseY;

    bool showFPS;

	SpotLight light;

	Vertex position;
	float Wx, Wy, Wz;
    __int64 cpuHz;

	TextureHandler *texHandler;
	TextureID text, wood, woodBump, particle, lamp;


	ParticleSystem *particleSystem;

	
	BOOL PreCreateWindow(CREATESTRUCT &cs);
	int  OnCreate(LPCREATESTRUCT lpCreateStruct);
	void OnSize(UINT nType, int cx, int cy);
	void OnClose();

	void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnKeyUp  (UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

	void OnLButtonDown(UINT nFlags, CPoint point);
	void OnLButtonUp  (UINT nFlags, CPoint point);
	void OnMouseMove  (UINT nFlags, CPoint point);

	void OnKillFocus(CWnd *pNewWnd);

	DECLARE_MESSAGE_MAP ()
};

BEGIN_MESSAGE_MAP(OpenGLWindow, CFrameWnd)
	
	ON_WM_CREATE ()
	ON_WM_CLOSE ()

	ON_WM_KEYDOWN ()
	ON_WM_KEYUP ()
	ON_WM_SYSKEYDOWN ()

	ON_WM_LBUTTONDOWN ()
	ON_WM_LBUTTONUP ()
	ON_WM_MOUSEMOVE ()

	ON_WM_KILLFOCUS ()

END_MESSAGE_MAP()


/* ################################################################################################################### */

void OpenGLWindow::DrawFrame(){
    static bool first = true;

	int i;
    static __int64 prev, curr;
    float time, speed;
    static float currTime = 0;

    prev = curr;
    curr = getCycleNumber();
    if (first){
        prev = curr;
        first = false;
    }
    time = (float) (double(curr - prev) / cpuHz);
	speed = time * 1500;

    currTime += time;


	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(75, (GLdouble) Width / (GLdouble) Height, 1, 32000);


    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glRotatef(Wz, 0.0f, 0.0f, 1.0f);
    glRotatef(Wx, 1.0f, 0.0f, 0.0f);
    glRotatef(Wy, 0.0f, 1.0f, 0.0f);

	glScalef(1,1,-1);

	glTranslatef(-position.x, -position.y, -position.z);


	float modelView[16];
	float projection[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,  modelView);
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	Vertex dx(modelView[0], modelView[4], modelView[8]);
	Vertex dy(modelView[1], modelView[5], modelView[9]);
	Vertex dz(modelView[2], modelView[6], modelView[10]);

	Vertex dir(0,0,0);

	if (Key[VK_LEFT ]) dir -= dx;
	if (Key[VK_RIGHT]) dir += dx;

	if (Key[VK_UP  ]) dir -= dz;
	if (Key[VK_DOWN]) dir += dz;

	if (Key[VK_SHIFT  ]) dir -= dy;
	if (Key[VK_CONTROL]) dir += dy;

	if (Key[VK_ENTER]){
		position = Vertex(640,500,-450);
		
		Wx = 30;
		Wy = -48;
		Wz = 0;
	}

	if (lengthSqr(dir) != 0){
		dir.normalize();
	}


	Vertex oldPos = position;
	position += speed * dir;

	particleSystem->update(currTime);



	light.setPosition(Vertex(-400 * sinf(currTime), 400, -400 * cosf(currTime)));
	light.setOrientation(-20 + 10 * cosf(currTime * 0.73f), (0.8f * cosf(currTime) - currTime) * 180 / PI, currTime * 25);



	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);


	texHandler->setTextures(lamp);
	texHandler->apply();

	glColor3f(1,1,1);
	light.draw();




	glDepthMask(GL_FALSE);

	static Vertex vertices[4] = {
		Vertex(-2048, 0,  2048), 
		Vertex( 2048, 0,  2048), 
		Vertex( 2048, 0, -2048), 
		Vertex(-2048, 0, -2048)
	};
	static float scale = 3;


	texHandler->setTextures(0, woodBump, 1, 2);
	texHandler->setTexEnvs(TEXENV_DOT3_NEXT, TEXENV_MODULATE_PREV_NEXT, TEXENV_MODULATE_PREV_NEXT, TEXENV_ADD_PRIMARY);
	texHandler->apply();

	glColor3f(0.05f,0.05f,0.05f);

	static GLfloat s[4] = {1.0f, 0.0f, 0.0f, 0.0f};
	static GLfloat t[4] = {0.0f, 1.0f, 0.0f, 0.0f};
	static GLfloat r[4] = {0.0f, 0.0f, 1.0f, 0.0f};
	static GLfloat q[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	
	texHandler->setActiveTexture(2);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r);
	glTexGenfv(GL_Q, GL_OBJECT_PLANE, q);
	
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);

	texHandler->setActiveTexture(3);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r);
	glTexGenfv(GL_Q, GL_OBJECT_PLANE, q);
	
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);

	glMatrixMode(GL_TEXTURE);
	
	light.setupSpotLightProjection(texHandler, 2, 0.005f);
	
	glBegin(GL_QUADS);
	for (i = 0; i < 4; i++){
		Vertex p = (light.getPosition() - vertices[i]) * 0.5f / light.getSize();
		glMultiTexCoord3f(GL_TEXTURE0_ARB, p.x + 0.5f, -p.z + 0.5f, p.y + 0.5f);
		glMultiTexCoord2f(GL_TEXTURE1_ARB, scale * (i == 1 || i == 2), scale * (i >= 2));
		
		glVertex3fv((GLfloat *) (vertices + i));
	}
	glEnd();

	texHandler->setActiveTexture(3);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_GEN_Q);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();

	texHandler->setActiveTexture(2);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_GEN_Q);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();





	texHandler->setTextures(particle);
	texHandler->setBlendMode(GL_FUNC_REVERSE_SUBTRACT_EXT);
	texHandler->setBlending(GL_ONE, GL_ONE);
	texHandler->apply();


	Matrix m;
	m.loadShadowMatrix(Plane(0,1,0,0), light.getPosition());

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glMultMatrixf((GLfloat *) &m);


	Vertex ldx, ldy, ldz;
	light.getSpace(ldx, ldy, ldz);

	particleSystem->draw(ldx, ldy);

	glPopMatrix();










	glDepthMask(GL_FALSE);


	texHandler->setTextures(wood);
	texHandler->setBlending(GL_DST_COLOR, GL_SRC_COLOR);
	texHandler->apply();

	glColor3f(1,1,1);

	glBegin(GL_QUADS);
	for (i = 0; i < 4; i++){
		glTexCoord2f(scale * (i == 1 || i == 2), scale * (i >= 2));
		glVertex3fv((GLfloat *) (vertices + i));
	}
	glEnd();


	texHandler->setTextures(1, 2, particle);
	texHandler->setTexEnvs(TEXENV_MODULATE_NEXT, TEXENV_ADD_PRIMARY, TEXENV_MODULATE);
	texHandler->setBlending(GL_ONE_MINUS_DST_COLOR, GL_ONE);
	texHandler->apply();

	texHandler->setActiveTexture(0);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r);
	glTexGenfv(GL_Q, GL_OBJECT_PLANE, q);
	
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);

	texHandler->setActiveTexture(1);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r);
	glTexGenfv(GL_Q, GL_OBJECT_PLANE, q);
	
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);

	glMatrixMode(GL_TEXTURE);
	
	light.setupSpotLightProjection(texHandler, 0, 0.005f);
	
	
	particleSystem->draw(dx, dy, 2);


	texHandler->setActiveTexture(1);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_GEN_Q);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();

	texHandler->setActiveTexture(0);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_GEN_Q);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	
	glDepthMask(GL_TRUE);



	if (showFPS){
		static float fps[8];
		char str[256];

		for (i = 0; i < 7; i++){
			fps[i] = fps[i+1];
		}

		fps[7] = 1.0f / time;
		float sum = 0;
		for (i = 0; i < 8; i++) sum += fps[i];
		
		sprintf(str,"%d", (int) (sum / 8 + 0.5f));

	
		glColor3f(1,1,1);
		
		texHandler->setTextures(text);
		texHandler->setBlending(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		texHandler->apply();

		glDrawText(str, 0.02f, 0.02f, 0.03f, 0.03f);

	}

	SwapBuffers(hdc);
}


OpenGLWindow::OpenGLWindow(){
	Width   = 1024;
	Height  = 768;
	Bpp     = 32;
	Refresh = 85;
	fullScreen = true;
	captureMouse = true;

	mouseSensibility = 0.1f;
	invertMouse = true;

	middleX = Width / 2;
	middleY = Height / 2;
	setResolution(Width, Height, Bpp, Refresh);

    SetCursorPos(middleX, middleY);
	
    showFPS = true;

	position = Vertex(640,500,-450);
	
	Wx = 30;
	Wy = -48;
	Wz = 0;

	ZeroMemory(&Key, sizeof(Key));
	leftButton = false;

	CRect r(0, 0, Width, Height);
	windowedRect = r;

	Create(NULL, "OpenGL", WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, r);
}

BOOL OpenGLWindow::PreCreateWindow(CREATESTRUCT &cs){
	CFrameWnd::PreCreateWindow(cs);

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	return 1;
}


int OpenGLWindow::OnCreate(LPCREATESTRUCT lpCreateStruct){
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1) return -1;

	ShowWindow(SW_SHOW);
	SetForegroundWindow();
	SetFocus();

	hdc = ::GetDC((HWND) *this);

    static PIXELFORMATDESCRIPTOR pfd = {
        sizeof (PIXELFORMATDESCRIPTOR),
			1,
			PFD_DRAW_TO_WINDOW |
			PFD_SUPPORT_OPENGL |
			PFD_DOUBLEBUFFER,
			PFD_TYPE_RGBA,
			Bpp,
			0, 0, 0, 0, 0, 0,
			0, 0,
			0, 0, 0, 0, 0,
			24,
			0,
			0,
			PFD_MAIN_PLANE,
			0,
			0, 0, 0
    };

    int nPixelFormat = ChoosePixelFormat(hdc, &pfd);
    SetPixelFormat(hdc, nPixelFormat, &pfd);

	hrc = wglCreateContext(hdc);
	wglMakeCurrent(hdc,hrc);
	
	ShowCursor(FALSE);

    initCPU();
    cpuHz = getHz();

	initExtensions(hdc);

	CString supported, missing;
    if (GL_ARB_texture_env_dot3)     supported += "GL_ARB_texture_env_dot3\n";   else missing += "GL_ARB_texture_env_dot3\n";
    if (GL_ARB_texture_env_crossbar || GL_NV_texture_env_combine4)  supported += "GL_ARB_texture_env_crossbar/GL_NV_texture_env_combine4\n";  else missing += "GL_ARB_texture_env_crossbar/GL_NV_texture_env_combine4\n";
	if (GL_EXT_blend_subtract)       supported += "GL_EXT_blend_subtract\n";     else missing += "GL_EXT_blend_subtract\n";
    if (GL_EXT_texture3D)            supported += "GL_EXT_texture3D\n";          else missing += "GL_EXT_texture3D\n";
    if (GL_EXT_texture_edge_clamp)   supported += "GL_EXT_texture_edge_clamp\n"; else missing += "GL_EXT_texture_edge_clamp\n";

    if (missing.GetLength() > 0){
        MessageBox("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extension aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions", MB_OK);
        texHandler = NULL;
        PostMessage(WM_CLOSE);
        return 0;
    }

	texHandler = new TextureHandler();

	texHandler->createVectorField(64);
	//texHandler->createSpotLightMap(64);
	texHandler->loadFromFile("..\\Textures\\Spotlight.png");
	texHandler->setWrapMode(1, GL_CLAMP_TO_EDGE_EXT, GL_CLAMP_TO_EDGE_EXT);

	unsigned char cutOff[2] = {0, 255};
	texHandler->loadFromMemory(cutOff, sizeof(cutOff), 1, 1);
	texHandler->setFilter(2, GL_NEAREST);
	texHandler->setWrapMode(2, GL_CLAMP_TO_EDGE_EXT, GL_CLAMP_TO_EDGE_EXT);

	text       = texHandler->loadFromFile("..\\Textures\\font2.dds");
	particle   = texHandler->loadFromFile("..\\Textures\\Particle.png");
	texHandler->setWrapMode(particle, GL_CLAMP_TO_EDGE_EXT, GL_CLAMP_TO_EDGE_EXT);

	wood       = texHandler->loadFromFile("..\\Textures\\Wood.dds");
	woodBump   = texHandler->loadFromFile("..\\Textures\\WoodBump.png");
	texHandler->toNormalMap(woodBump);
	texHandler->setInternalFormat(woodBump, GL_RGB5);

	lamp       = texHandler->loadFromFile("..\\Textures\\metalvent2.jpg");

	texHandler->uploadAll();


	particleSystem = new ParticleSystem();
	particleSystem->setColorScheme(COLOR_SCHEME_SMOKE);
	particleSystem->setSpawnRate(80);
	particleSystem->setSpeed(300, 50, 0.3f, 0.3f);
	particleSystem->setLife(5.5f, 0);
	particleSystem->setDirectionalForce(Vertex(0,-10,0));
	particleSystem->setFrictionFactor(0.7f);
	particleSystem->setPosition(Vertex(0,50,0));
	particleSystem->setSize(50, 15);

	light.setFov(100);
	light.setSize(5800);

	
	return 0;
}

void OpenGLWindow::OnSize(UINT nType, int cx, int cy){
	glViewport(0, 0, cx, cy);
}

void OpenGLWindow::OnClose(){
	if (texHandler != NULL){
		texHandler->unloadAll();
		delete texHandler;

		delete particleSystem;
	}

	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hrc);
	::ReleaseDC((HWND) *this,hdc);

	if (fullScreen) resetResolution();

	ShowCursor(TRUE);
	DestroyWindow();
}


void OpenGLWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags){
	Key[nChar & 0xFF] = true;

    switch(nChar){
        case ' ':
            showFPS = !showFPS;
            break;
        case VK_ESCAPE:
            if (!fullScreen && captureMouse){
                ShowCursor(TRUE);
                captureMouse = false;
            } else {
                PostMessage(WM_CLOSE, 0,0);
            }
            break;
    }
}

void OpenGLWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags){
	Key[nChar & 0xFF] = false;
}

void OpenGLWindow::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags){
	if (((nFlags >> 13) & 0x3) == 1 && nChar == VK_ENTER){
		if (fullScreen){
			resetResolution();
			middleX = (windowedRect.left + windowedRect.right)  / 2;
			middleY = (windowedRect.top  + windowedRect.bottom) / 2;
			MoveWindow(windowedRect, TRUE);

			fullScreen = false;
		} else {
			middleX = Width  / 2;
			middleY = Height / 2;
			MoveWindow(0, 0, Width, Height, TRUE);
			setResolution(Width, Height, Bpp, Refresh);

			captureMouse = true;
			ShowCursor(FALSE);

			fullScreen = true;
		}
	}
}

void OpenGLWindow::OnLButtonDown(UINT nFlags, CPoint point){
	mouseX = point.x;
	mouseY = point.y;

	windowMoved = false;
	leftButton = true;
}

void OpenGLWindow::OnLButtonUp(UINT nFlags, CPoint point){
	if (!captureMouse){
		if (!windowMoved){
			captureMouse = true;
			ShowCursor(FALSE);
		}
	}

	leftButton = false;
}

void OpenGLWindow::OnMouseMove(UINT nFlags, CPoint point){
	static bool changeMouse = false;

	if (captureMouse){
		ClientToScreen(&point);

		Wx += mouseSensibility * (point.y - middleY) * (invertMouse? -1 : 1);
		Wy += mouseSensibility * (point.x - middleX);
		
		if (changeMouse = !changeMouse) SetCursorPos(middleX, middleY);
	} else {
		if (leftButton){
			windowedRect += CSize(point.x - mouseX, point.y - mouseY);
			middleX = (windowedRect.left + windowedRect.right)  / 2;
			middleY = (windowedRect.top  + windowedRect.bottom) / 2;

			MoveWindow(windowedRect, TRUE);
			windowMoved = true;
		}
	}
}

void OpenGLWindow::OnKillFocus(CWnd *pNewWnd){
	if (fullScreen){
		resetResolution();
		middleX = (windowedRect.left + windowedRect.right)  / 2;
		middleY = (windowedRect.top  + windowedRect.bottom) / 2;
		MoveWindow(windowedRect, TRUE);
		fullScreen = false;
		captureMouse = false;
		ShowCursor(TRUE);
	}
}

/* ################################################################################################################### */


class OpenGLApp : public CWinApp {
public:
    virtual BOOL InitInstance();
	BOOL OnIdle(LONG lCount);
};


BOOL OpenGLApp::InitInstance(){
	m_pMainWnd = new OpenGLWindow();

	return TRUE;
}

BOOL OpenGLApp::OnIdle(LONG lCount){
	
	((OpenGLWindow *) m_pMainWnd)->DrawFrame();

	return TRUE;
}

OpenGLApp openGLApp;


/* ########################################## End of file ############################################################ */
